import os

from redis import Redis
from rq import Worker, Queue, Connection
from typing import List

DEFAULT_CONNECTION_URL = os.getenv('REDIS_URL', 'redis://localhost:6379')

class TaskMaster:
    def __init__(self, connection_url: str = DEFAULT_CONNECTION_URL, channel_subscriptions: List[str] = None):
        """
        Initialize the TaskMaster with the given connection URL.
        """
        self.connection = Redis.from_url(connection_url)
        self.job_queue = Queue(connection=self.connection)
        self.channel_subscriptions = channel_subscriptions or []

    def work(self):
        """
        Start the worker and process tasks from the queue.
        """
        with Connection(self.connection):
            print("Connected to Redis")
            worker = Worker([self.job_queue])
            print("Worker created")
            worker.work()
            print("Worker working")

# Create a queue
q = Queue(connection=Redis.from_url(DEFAULT_CONNECTION_URL))

# Create a connection to the Redis server
REDIS_CONNECTION_URL = os.getenv('REDIS_URL', 'redis://localhost:6379')
worker_listen: List[str] = ["high", "default", "low"]

# Create a worker
worker = TaskMaster(REDIS_CONNECTION_URL,worker_listen)



from dotenv_vault import load_dotenv
load_dotenv()

def run_tests():
    print("Running tests...")
    # TODO: run tests
    print("Tests complete.")
    sys.exit(0)


if __name__ == "__main__":
    run_tests()





import argparse
from dash import Dash, dcc, html, Input, Output, callback

external_stylesheets = ['https://codepen.io/chriddyp/pen/bWLwgP.css']

app = Dash(__name__, external_stylesheets=external_stylesheets)

server = app.server

app.layout = html.Div([
    html.H1('Hello World'),
    dcc.Dropdown(['LA', 'NYC', 'MTL'],
        'LA',
        id='dropdown'
    ),
    html.Div(id='display-value')
])

@callback(Output('display-value', 'children'), Input('dropdown', 'value'))
def display_value(value):
    return f'You have selected {value}'

if __name__ == '__main__':
    print("Starting dashboard app server")
    debug = False
    if argparse.argv[1] == "debug":
        print("Debug mode enabled for dashboard app server")
        debug = True
    app.run(debug=debug)



"""
This module contains the main application for streaming trades from Polygon and Unusual Whales APIs.

It sets up logging, initializes the necessary handlers, and runs the main application loop.
"""
import asyncio

from polygon.websocket import WebSocketClient
from polygon.websocket.models import Market, Feed

from tradestream.services.api import EquityTradeHandler, OptionOrderFlowHandler, MongoDBHandler, QuoteHandler

from tradestream.services.models import Config

class TradesStreamApp:
    """ 
    Main application class for streaming trades from Polygon and Unusual Whales APIs.

    This class initializes the necessary handlers and provides a method to run the application.

    Attributes:
        config (Config): Configuration object loaded from the config file.
        db_handler (MongoDBHandler): Handler for MongoDB operations.
        order_flow_handler (OptionOrderFlowHandler): Handler for Unusual Whales API.
        equity_trade_handler (EquityTradeHandler): Handler for Polygon API.
    """

    def __init__(self, config: Config, feed, market, subscriptions):
        """
        Initialize the TradesStreamApp.

        Args:
            config (Config): Configuration object initialized by yaml file.
        """
        print("Initializing TradesStreamApp")
        self.config = config
        self.db_handler = MongoDBHandler(self.config)
        self.order_flow_handler = OptionOrderFlowHandler(self.config, self.db_handler)
        self.quote_handler = QuoteHandler(self.config, self.db_handler)
        self.equity_trade_handler = EquityTradeHandler(self.config, self.db_handler, self.quote_handler)
        self.polygon_websocket_client = WebSocketClient(
            api_key=self.config.polygon_api_key,
            feed=feed,
            market=market,
            verbose=True,
            subscriptions=subscriptions,
        )

    async def run(self):
        """
        Run the main application loop.

        This method starts both the Unusual Whales and Polygon handlers concurrently.
        """
        try:
            await asyncio.gather(
                self.order_flow_handler.run(),
                self.polygon_websocket_client.connect(self.equity_trade_handler.add),
                self.equity_trade_handler.run(),
                self.quote_handler.start_processing_api_calls(),
            )
        except Exception as e:
            print(f"Error in event stream: {e}")


async def main():
    """
    Main entry point for the application.
    """
    websocket_subscriptions:list[str] = []
    config = Config()
    websocket_subscriptions.append("T.*")
    watchlist = config.equity_watchlist
    if watchlist:
        for sym in watchlist:
            # websocket_subscriptions.append(f"Q.{sym}")
            # websocket_subscriptions.append(f"AM.{sym}")
            print(f"Subscribing to trades, quotes, and aggregate minute bars for {sym}.")
    app = TradesStreamApp(
        config,
        feed=Feed.RealTime,
        market=Market.Stocks,
        subscriptions=websocket_subscriptions
    )
    await app.run()
    print("TradesStreamApp running")

def unsubscribe_all():
    print("unsubscribing all")
    pass

def start_worker():
    print("starting daemon")
    asyncio.run(main())

def stop_worker():
    print("stopping daemon")
    unsubscribe_all()
    print("daemon stopped")
    pass
    
if __name__ == "__main__":
    asyncio.run(main())
